﻿using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using Infragistics.Controls.Grids;
using Infragistics.Persistence;
using Infragistics.Persistence.Primitives;

namespace XamGridAddingColumnPersistedConfig
{
    public static class ConfigManager
    {
        public static void Initialize(XamGrid grid)
        {
            if (File.Exists("Configs.xml"))
            {
                var text = File.ReadAllText("Configs.xml");
                using (var stringReader = new StringReader(text))
                {
                    var xmlSerializer = new XmlSerializer(typeof(List<Configs>));
                    Configs = (List<Configs>) xmlSerializer.Deserialize(stringReader);
                    LoadConfig(Configs.First(x => x.Name == "Standard"), grid);
                }
            }
            else
            {
                Configs = new List<Configs> { new Configs { Name = "Standard" } };
                SaveConfig(Configs[0], grid);
            }
        }

        public static void LoadConfig(Configs config, XamGrid grid)
        {
            MemoryStream bigStreamOut;

            var inputStream = new MemoryStream(config.ConfigBytes);

            using (bigStreamOut = new MemoryStream())
            {
                using (var bigStream = new GZipStream(inputStream, CompressionMode.Decompress))
                    bigStream.CopyTo(bigStreamOut);

                bigStreamOut.Position = 0;
                PersistenceManager.Load(grid, bigStreamOut);
            }
        }

        public static List<Configs> Configs { get; set; }

        private static PersistenceSettings _settings;

        private static PersistenceSettings Settings
        {
            get
            {
                if (_settings != null) return _settings;

                _settings = new PersistenceSettings
                {
                    SavePersistenceOptions = PersistenceOption.OnlySpecified,
                    LoadPersistenceOptions = PersistenceOption.OnlySpecified
                };

                var propertyPathList = new List<string> {
                    "ColumnLayouts[].Columns[].Width",
                    "ColumnLayouts[].Columns[].Key",
                    "ColumnLayouts[].Columns[].Visibility",
                    "ColumnLayouts[].Columns[].IsGroupBy",
                    "ColumnLayouts[].Columns[].IsSorted",
                    "FilteringSettings.AllowFiltering",
                    "FilteringSettings.FilterMenuSelectionListGeneration",
                    "FilteringSettings.FilterMenuCustomFilteringButtonVisibility" };

                foreach (var propertyPath in propertyPathList)
                {
                    _settings.PropertySettings.Add(new PropertyNamePersistenceInfo
                    {
                        PropertyName = propertyPath,
                        Options = PropertyNamePersistenceOptions.PropertyPath
                    });
                }


                return _settings;
            }
        }

        public static void SaveConfig(Configs comboBoxSelectedItem, XamGrid xamGrid)
        {
            MemoryStream mStream = null;
            MemoryStream outStream = null;
            byte[] result;

            try
            {

                mStream = PersistenceManager.Save(xamGrid, Settings);
                outStream = new MemoryStream();
                using (var tinyStream = new GZipStream(outStream, CompressionMode.Compress))
                    mStream.CopyTo(tinyStream);
                result = outStream.ToArray();
            }
            finally
            {
                outStream?.Dispose();

                mStream?.Dispose();
            }

            comboBoxSelectedItem.ConfigBytes = result;

            Save();
        }

        public static Configs RemoveConfig(Configs comboBoxSelectedItem, XamGrid grid)
        {
            if (comboBoxSelectedItem.Name == "Standard")
                return comboBoxSelectedItem;

            var config = Configs.First(x => x.Name == "Standard");
            LoadConfig(config, grid);
            Configs.Remove(comboBoxSelectedItem);
            Save();
            return config;
        }

        private static void Save() 
        {
            using (var stringWriter = new StringWriter(new StringBuilder()))
            {
                var xmlSerializer = new XmlSerializer(typeof(List<Configs>));
                xmlSerializer.Serialize(stringWriter, Configs);
                var newFile = stringWriter.ToString();
                File.WriteAllText("Configs.xml", newFile);
            }
        } 
    }
}
